# This file contains scripts for annotating the sponge intersection network
# and also contains all the anuran commands used for the sponge networks.

import networkx as nx
import pandas as pd
import glob
from shutil import copy2, rmtree
from os import popen
import os
from manta.main import cluster_graph
import multiprocessing as mp
from functools import partial
from subprocess import call
import csv
import ast
import numpy as np
import pandas as pd
from scipy.stats import chi2_contingency
from collections import Counter
loc = os.getcwd() + '//globalsponge_data//'

###########################################
# construct order-specific CoNet networks #
# Run flashweave_script.jl from Atom      #
# Final network files in supplements      #
###########################################
files = ['Axinellida', 'Chondrillida', 'Clionaida',
         'Dictyoceratida', 'Haplosclerida', 'Plakinidae',
         'Poecilosclerida', 'Suberitida', 'Tetractinellida',
         'Verongiida']
libpath = 'path_to_conet_folder' + '//lib//CoNet.jar'
pool = mp.Pool(6)
call_list = []
for file in files:
    # count table from processed
    tabname = loc + 'host_order//' + file + '_otu.txt'
    graphname = loc + 'host_order//' + file + '_conet.tsv'
    cmd = loc +'CoNet_script.sh' + ' ' + tabname + ' ' + ' ' + graphname + ' ' + libpath + \
                      ' ' + loc + 'host_order//' + ' ' + '1000'
    call_list.append(cmd)
# run all network inference in parallel
func = partial(call, shell=True)
results = pool.map(func, iter(call_list))

###### Processed networks are included in the networks folder
###### The scripts below are for parsing raw data (not included)
###### CoNet and FlashWeave networks need to be converted to graphml files with taxonomy
for file in files:
    conet = loc + 'host_order//' + file + '_conet.tsv'
    with open(conet, 'r') as fin:
        data = fin.read().splitlines(True)
        fin.close()
    data = data[2:]
    # take median of edge signs
    # since edge weight is arbitrary
    weight = [b[0] for b in csv.reader(data, delimiter='\t')]
    weight = [word.replace('mutualExclusion', '-1') for word in weight]
    weight = [word.replace('copresence', '1') for word in weight]
    # remove sign info from methods that do not assign it
    weight = [word.replace('unknown', 'None') for word in weight]
    weight = [ast.literal_eval(b) for b in weight]
    # remove none
    weight = [[elem for elem in sublist if elem is not None] for sublist in weight]
    weight = [np.median(x) for x in weight]
    network = [b[15] for b in csv.reader(data, delimiter='\t')]
    network = [b.split('->') for b in network]
    for i in range(len(network)):
        network[i].append(str({'weight': weight[i]}))
        network[i] = ' '.join(network[i])
    conet = nx.parse_edgelist(network)
    flashweave = loc + 'host_order//' + file + '_flash.gml'
    flashweave = nx.read_gml(flashweave)
    #### add taxonomy
    taxonomy = pd.read_csv(loc + 'host_order//' + file + '_tax.txt', sep='\t')
    # remove nan values, breaks the graphml format
    taxonomy = taxonomy.fillna(value='None')
    for col in ['Rank1', 'Rank2', 'Rank3', 'Rank4', 'Rank5', 'Rank6']:
        tax_dict = dict(zip(taxonomy['#otu '], taxonomy[col]))
        nx.set_node_attributes(conet, tax_dict, col)
        nx.set_node_attributes(flashweave, tax_dict, col)
    conet = nx.to_undirected(conet)
    flashweave = nx.to_undirected(flashweave)
    nx.write_graphml(conet, loc + 'conet//' + file + '_conet.graphml')
    nx.write_graphml(flashweave, loc + 'flashweave//' + file + '_flash.graphml')

#### anuran script for conet networks
# anuran -i C://Users//u0118219//Documents//anuran_supplements//globalsponge_data//conet_networks -o C://Users//u0118219//Documents//anuran_supplements//globalsponge_data//globalsponge -size 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1  -cs 0.05 0.2 -prev 0.2 0.5 -perm 10 -gperm 20 -nperm 50 -c -stats fdr_bh -core 4

network = nx.read_graphml(loc + 'globalsponge_core_conet_networks_0.3_intersection.graphml')
##### cross-reference HMA-LMA results from Moitinho et al 2017
# https://www.frontiersin.org/articles/10.3389/fmicb.2017.00752/full#B23
# parsed one of the supplementary tables so it is in a long format
# the original table is Supplementary Table 5
data = pd.read_excel(loc + 'Moitinho2017_table5_long.xlsx')
phylum = data[data['Level'] == 'Phylum']
for node in network.nodes:
    tax = network.nodes[node]
    for level in tax:
        try:
            tax[level] = tax[level].split('__')[1]
        except IndexError:
            tax[level] = 'None'
    if 'Rank2' in tax:
        if tax['Rank2'] in phylum['Taxonomy'].tolist():
            vals = phylum[phylum['Taxonomy'] == tax['Rank2']]
            if vals[vals['Type'] == 'HMA']['Average'].iloc[0] > vals[vals['Type'] == 'LMA']['Average'].iloc[0]:
                sponge_assoc = 'HMA'
            else:
                sponge_assoc = 'LMA'
            tax['Sponge'] = sponge_assoc
nx.write_graphml(network, loc + 'Sponge_3_intersection.graphml')


##### run manta on 3 intersection
results = cluster_graph(network, limit=2, max_clusters=4,
                        min_clusters=2, min_cluster_size=0.2,
                        iterations=200, subset=0.8,
                        ratio=0.8, edgescale=0.8,
                        permutations=200, verbose=False)[0]
clusters = nx.get_node_attributes(results, 'cluster')
nx.set_node_attributes(network, clusters, name='Cluster')
nx.write_graphml(network, loc + 'Sponge_3_intersection.graphml')

##### check if manta clusters contain more HMA-LMA associations than expected by chance
vals = []
for key in network.nodes:
    try:
        vals.append([network.nodes[key]['Sponge'],
                    network.nodes[key]['Cluster']])
    except KeyError:
        vals.append(['None',
                    network.nodes[key]['Cluster']])
vals = Counter(map(tuple, vals))
# convert to scipy stats format
# 3 rows for HMA-LMA-None, 2 cols for clust
# leaving out taxa with no assignment
freq = np.array([[vals[('HMA', 0.0)], vals[('HMA', 1.0)], vals[('HMA', 2.0)]],
                [vals[('LMA', 0.0)], vals[('LMA', 1.0)], vals[('HMA', 2.0)]]])
# possibly repeat with class assignments
chi2_contingency(freq)  # p-value <0.05

##### compare tools
# first move flashweave and conet networks to same folders
network_list = ['Axinellida', 'Chondrillida',
                'Clionaida', 'Dictyoceratida',
                'Haplosclerida', 'Plakinidae',
                'Poecilosclerida', 'Suberitida',
                'Tetractinellida', 'Verongiida']

for network in network_list:
    os.mkdir(loc + network)
    conet = loc + 'conet_networks//' + network + '_conet.graphml'
    copy2(conet, loc+network)
    flashweave = loc + 'flashweave_networks//' + network + '_flash.graphml'
    copy2(flashweave, loc+network)

# compare each FlashWeave with each conet network
# moved every matching flashweave and CoNet network to- their own folder
for network in network_list:
    anuran_command = ('anuran -i ' + network + ' -o ' + network + ' -size 1 -perm 5 -nperm 5 -core 6')
    print(anuran_command)
    #popen(anuran_command)

all_sets = pd.DataFrame(columns=['Unnamed: 0', 'Conserved fraction', 'Group', 'Network', 'Network type',
                                 'Prevalence of conserved fraction', 'Samples', 'Set size', 'Set type',
                                 'Set type (absolute)', 'Order'])
for network in network_list:
    data = pd.read_csv(loc + network + "_sets.csv")
    data['Order'] = network
    all_sets = all_sets.append(data)

for network in network_list:
    rmtree(loc + network)

all_sets.to_csv(loc + "order_sets.csv")


##### check if FlashWeave edges are more / less likely to be in the core
##### than FlashWeave + conet edges
# run anuran on matches between Flashweave and conet networks