library(ggplot2)
library(igraph)
setwd("~/anuran_supplements/globalsponge_data")

data <- read.csv("order_sets.csv")
data <- data[,4:length(colnames(data))]
data$Network <- factor(data$Network, levels=c("Random", "Degree", "Input"))
difference <- data[data$Set.type == 'Difference',]
ggplot(difference, aes(x=Order, y=Set.size, fill=Network)) + 
  geom_bar(stat="identity", position="dodge", width=0.4) + 
  theme_minimal() + theme(axis.text.x = element_text(angle=90, hjust=1), legend.position='none') + scale_fill_viridis_d() + coord_flip() + 
  labs(title="Difference", x='Sponge host order', y='Size of difference')
ggsave(paste("../Figures/Difference_sponges_order.pdf", sep="_"), width = 3, height=3, units="in")
Sys.sleep(5)

intersection <- data[data$Set.type == 'Intersection 1',]
ggplot(intersection, aes(x=Order, y=Set.size, fill=Network)) + 
  geom_bar(stat="identity", position="dodge", width=0.4) + 
  theme_minimal() + theme(axis.text.x = element_text(angle=90, hjust=1), legend.position="none") + scale_fill_viridis_d() + coord_flip() + 
  labs(title="Intersection", x='Sponge host order', y='Size of intersection')
ggsave(paste("../Figures/Intersection_sponges_order.pdf", sep="_"), width = 3, height=3, units="in")
Sys.sleep(5)

# make figure of input network sizes
orders <- unique(data$Order)
data <- data.frame(matrix(ncol=3, nrow=0))
colnames(data) <- c("Group", "Network.size", "Network")
i = 1
setwd("~/anuran_supplements/globalsponge_data/conet_networks")
for (level in orders){
  setwd('../flashweave_networks')
  flash = read_graph(paste(level, "_flash.graphml", sep=""), format="graphml")
  setwd('../conet_networks')
  conet = read_graph(paste(level, "_conet.graphml", sep=""), format="graphml")
  data[i,] <- c(level, length(E(flash)), "FlashWeave")
  i = i + 1
  data[i,] <- c(level, length(E(conet)), "CoNet")
  i = i + 1
}

setwd('..')
data$Network.size <- as.numeric(data$Network.size)
ggplot(data, aes(x=Group, y=Network.size, fill=Network)) + 
  geom_bar(stat="identity", position="dodge", width=0.4) + 
  theme_minimal() + theme(axis.text.x = element_text(angle=90, hjust=1), legend.position="none") + 
  scale_fill_viridis_d(begin=0.2, end=0.9) + coord_flip() + 
  labs(title="", y='Number of edges in network', x='Sponge host order')
ggsave(paste("Networks_sponges_order.pdf", sep="_"), width = 3, height=3, units="in")
Sys.sleep(5)

