library(igraph)
library(ggplot2)

setwd("~/anuran_supplements/simulation_data")
files <- c("no_core_subsampled_sets.csv", "core_1_subsampled_sets.csv", "core_2_subsampled_sets.csv", "core_3_subsampled_sets.csv", "core_4_subsampled_sets.csv")
#cores <- c("No core", "Core 1", "Core 2", "Core 3", "Core 4")
cores <- c("0s0p", "20s33p", "20s50p", "20s33p\n30s50p", "50s10p\n30s33p\n20s50p")
data <- read.csv(files[[1]])
data$core <- cores[[1]]
for (i in 2:5){
  new_data <- read.csv(files[[i]])
  new_data$core <- cores[[i]]
  data <- rbind(data, new_data)
}
drop <- c("X", "Conserved.fraction", "Prevalence.of.conserved.fraction")
data <- data[, !(names(data) %in% drop)]
data$Network <- factor(data$Network, levels=c("Input", "Degree", "Random"))
data$core <- factor(data$core, levels=c("0s0p", "20s33p", "20s50p", "20s33p\n30s50p", "50s10p\n30s33p\n20s50p"))
sets <- unique(data$Set.type)

# first look at connectivity
data <- data[data$Group == 0.05,]

for (i in 1:length(sets)){
  difference <- data[data$Set.type %in% sets[[i]],]
  if (i == 1){
    name = 'Size of difference'
  }
  else{
    name = 'Size of intersection'
  }
  ggplot(difference, aes(x=Samples, y=Set.size, colour=Network)) + 
    geom_smooth() + geom_point(alpha=0.1) + facet_grid(~core) + scale_x_continuous(limits=c(10, 50)) + 
    theme_minimal() + theme(axis.text.x = element_text(angle=90, hjust=1)) + scale_color_viridis_d() +
    labs(x='Number of networks', y=name)
  ggsave(paste(sets[[i]], "sim.pdf", sep="_"), width = 6, height=2.5, units="in")
  Sys.sleep(5)
}





